#version 330
#define GLSLIFY 1
// Common uniforms
uniform vec2 u_resolution;
uniform vec2 u_mouse;

// Texture uniforms
uniform sampler2D u_texture;

// Texture varyings
in vec2 v_texcoord;
out vec4 fragColor;

/*
 *  Calculates the Laplacian at a given texture position
 */
vec4 laplacian(vec2 uv, sampler2D tex, vec2 texture_size) {
    // Calculate the texture steps
    float du = 1.0 / texture_size.x;
    float dv = 1.0 / texture_size.y;

    // Calculate the laplacian
    vec4 lap = -texture(tex, uv);
    lap += 0.2 * texture(tex, uv + vec2(-du, 0.0));
    lap += 0.2 * texture(tex, uv + vec2(du, 0.0));
    lap += 0.2 * texture(tex, uv + vec2(0.0, -dv));
    lap += 0.2 * texture(tex, uv + vec2(0.0, dv));
    lap += 0.05 * texture(tex, uv + vec2(-du, -dv));
    lap += 0.05 * texture(tex, uv + vec2(du, -dv));
    lap += 0.05 * texture(tex, uv + vec2(du, dv));
    lap += 0.05 * texture(tex, uv + vec2(-du, dv));

    return lap;
}

// Calculates the pixel color from the pixel chemical concentrations
vec4 calculate_color(vec2 concentrations) {
    return vec4(concentrations * vec2(0.05, 1.0), 0.0, 1.0);
}

// Calculates the pixel chemical concentrations from the pixel color
vec2 calculate_concentrations(vec4 color) {
    return color.rg / vec2(0.05, 1.0);
}

/*
 * The main program
 */
void main() {
    // Set the Gray-Scott reaction-diffusion simulation parameters
    float D_A = 0.8;
    float D_B = 0.4;
    float feed = 0.06 * v_texcoord.x;
    float kill = 0.035 + 0.03 * v_texcoord.x + (0.022 - 0.015 * v_texcoord.x) * v_texcoord.y;
    float dt = 1.0;

    vec2 texCoord = v_texcoord;

    // Calculate the chemical concentrations from the pixel color
    vec4 pixel_color = texture(u_texture, texCoord.xy);
    vec2 concentrations = calculate_concentrations(pixel_color);
    float A = concentrations.x;
    float B = concentrations.y;

    // Calculate the laplacian
    vec2 lap = calculate_concentrations(laplacian(texCoord.xy, u_texture, u_resolution));

    // Calculate the new chemical concentration values
    float dA = (D_A * lap.r - A * B * B + feed * (1.0 - A)) * dt;
    float dB = (D_B * lap.g + A * B * B - (kill + feed) * B) * dt;
    concentrations += vec2(dA, dB);

    // Modify the concentrations in the pixels under the mouse position
    if (length(gl_FragCoord.xy - u_mouse) < 5.0) {
        concentrations = vec2(0.0, 0.9);
    }

    // Fragment shader output
    fragColor = calculate_color(concentrations);
}
